package cz.muni.fi.pa165.controller;

import cz.muni.fi.pa165.model.BandManagerDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RequestMapping("/api/v1/band-manager")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Band manager Service API", version = "1.0",
                description = "API used for performing CRUD operations on Band Manager"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Band Manager Service", description = "APIs for managing band manager")
public interface BandManagerController {
    @PostMapping
    @Operation(summary = "Create new band manager")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<BandManagerDTO> createBandManager(@RequestBody @Valid BandManagerDTO bandManagerDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get band manager by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<BandManagerDTO> getBandManagerById(@PathVariable("id") long id);

    @PutMapping("/{id}")
    @Operation(summary = "Update existing band manager by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<BandManagerDTO> updateBandManagerById(@PathVariable("id") long id, @RequestBody @Valid BandManagerDTO bandManagerDTO);

    @DeleteMapping("/{id}")
    @Operation(summary = "Remove existing band manager by ID")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    ResponseEntity<BandManagerDTO> removeBandManagerById(@PathVariable("id") long id);


}
