package cz.muni.fi.pa165.controller;

import cz.muni.fi.pa165.data.entity.Proposal;
import cz.muni.fi.pa165.model.BandManagerDTO;
import cz.muni.fi.pa165.model.ProposalDTO;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.server.resource.authentication.BearerTokenAuthentication;
import org.springframework.security.oauth2.server.resource.introspection.OAuth2IntrospectionAuthenticatedPrincipal;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RequestMapping("/api/v1/proposals")
@OpenAPIDefinition(
        info = @Info(title = "PA 165 Proposal Service API", version = "1.0",
                description = "API used for creating proposals"),
        servers = {
                @Server(url = "/", description = "Default Server URL")
        }
)
@Tag(name = "PA165 Proposal Service", description = "APIs for creating proposals")
public interface ProposalController {
    @PostMapping
    @Operation(summary = "Create new proposal")
    @ResponseStatus(HttpStatus.CREATED)
    ResponseEntity<ProposalDTO> createProposal(@RequestBody @Valid ProposalDTO proposalDTO);

    @GetMapping("/{id}")
    @Operation(summary = "Get band membership proposal by ID")
    @ResponseStatus(HttpStatus.OK)
    ResponseEntity<ProposalDTO> getProposalById(@PathVariable("id") long id);

    @PostMapping("/{proposalId}/accept")
    @Operation(summary = "Accept membership proposal and create a new team-member")
    ResponseEntity<Void> acceptProposal(@PathVariable("proposalId") long proposalId);

    @PostMapping("/{proposalId}/reject")
    @Operation(summary = "Reject membership proposal")
    ResponseEntity<Void> rejectProposal(@PathVariable("proposalId") long proposalId);

    @GetMapping
    @Operation(summary = "Get band membership proposals for user by userID")
    ResponseEntity<List<ProposalDTO>> getProposalsByUserId(@RequestParam("userId") String userId);


}
