package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.entity.BandManager;
import cz.muni.fi.pa165.exception.BandManagerNotFoundException;
import cz.muni.fi.pa165.mapper.BandManagerMapper;
import cz.muni.fi.pa165.model.BandManagerDTO;
import cz.muni.fi.pa165.data.repository.BandManagerRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
public class BandManagerServiceImpl implements BandManagerService{
    private final BandManagerRepository bandManagerRepository;
    private final BandManagerMapper bandManagerMapper;

    @Autowired
    public BandManagerServiceImpl(BandManagerRepository bandManagerRepository, BandManagerMapper bandManagerMapper) {
        this.bandManagerRepository = bandManagerRepository;
        this.bandManagerMapper = bandManagerMapper;
    }

    @Override
    public BandManagerDTO getBandManagerByID(long id) {
        BandManager found = bandManagerRepository.findById(id).orElseThrow(
                () -> new BandManagerNotFoundException(String.format("Band manager with id %s was not found", id)));
        return bandManagerMapper.BandManagerToBandManagerDTO(found);
    }

    @Override
    public BandManagerDTO createBandManager(BandManagerDTO bandManagerDTO) {

        BandManager saved = bandManagerRepository.save(bandManagerMapper.BandManagerDTOToBandManager(bandManagerDTO));
        return bandManagerMapper.BandManagerToBandManagerDTO(saved);
    }

    @Override
    public void removeBandManagerByID(long id) {
        BandManager found = bandManagerRepository.findById(id).orElseThrow(
                () -> new BandManagerNotFoundException(String.format("Band manager with id %s was not found", id)));
        bandManagerRepository.delete(found);
    }

    @Override
    public BandManagerDTO updateBandManagerByID(BandManagerDTO bandManagerDTO) {
        Optional<BandManager> found = bandManagerRepository.findById(bandManagerDTO.getId());
        if (found.isEmpty()) {
            throw new BandManagerNotFoundException(String.format("Band manager with id %s was not found", bandManagerDTO.getId()));
        }
        BandManager updated = bandManagerRepository.save(bandManagerMapper.BandManagerDTOToBandManager(bandManagerDTO));
        return bandManagerMapper.BandManagerToBandManagerDTO(updated);
    }




}
