package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.entity.BandManager;
import cz.muni.fi.pa165.data.entity.Proposal;
import cz.muni.fi.pa165.data.repository.ProposalRepository;
import cz.muni.fi.pa165.exception.BandManagerNotFoundException;
import cz.muni.fi.pa165.exception.ProposalNotFoundException;
import cz.muni.fi.pa165.mapper.ProposalMapper;
import cz.muni.fi.pa165.model.ProposalDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class ProposalServiceImpl implements ProposalService{
    private final ProposalRepository proposalRepository;
    private final ProposalMapper proposalMapper;

    @Autowired
    public ProposalServiceImpl(ProposalRepository proposalRepository, ProposalMapper proposalMapper) {
        this.proposalRepository = proposalRepository;
        this.proposalMapper = proposalMapper;
    }

    @Override
    public ProposalDTO getProposalById(long id) {
        Proposal found = proposalRepository.findById(id).orElseThrow(
                () -> new ProposalNotFoundException(String.format("Proposal with id %s was not found", id))
        );
        return proposalMapper.proposalToProposalDTO(found);
    }

    @Override
    public List<ProposalDTO> getProposalsByUserId(long userId) {
        List<Proposal> found = proposalRepository.findAll()
                .stream()
                .filter(x -> x.getUserId() == userId)
                .toList();
        return found.stream().map(proposalMapper::proposalToProposalDTO).toList();
    }

    @Override
    public ProposalDTO createProposal(ProposalDTO proposalDTO) {
        Proposal saved = proposalRepository.save(proposalMapper.proposalDTOToProposal(proposalDTO));
        return proposalMapper.proposalToProposalDTO(saved);
    }
    //Might not be used
    @Override
    public ProposalDTO updateProposal(ProposalDTO proposalDTO) {
        Optional<Proposal> found = proposalRepository.findById(proposalDTO.getId());
        if (found.isEmpty()) {
            throw new ProposalNotFoundException(String.format("Proposal with id %s was not found", proposalDTO.getId()));
        }
        Proposal updated = proposalRepository.save(proposalMapper.proposalDTOToProposal(proposalDTO));
        return proposalMapper.proposalToProposalDTO(updated);
    }

    @Override
    public void removeProposalById(long id) {
        Proposal found = proposalRepository.findById(id).orElseThrow(
                () -> new ProposalNotFoundException(String.format("Proposal with id %s was not found", id))
        );
        proposalRepository.delete(found);
    }

    @Override
    public void resolveProposal(long id) {
        Proposal found = proposalRepository.findById(id).orElseThrow(
                () -> new ProposalNotFoundException(String.format("Proposal with id %s was not found", id))
        );
        found.setResolved(true);
        proposalRepository.save(found);
    }
}
