package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.entity.BandManager;
import cz.muni.fi.pa165.data.repository.BandManagerRepository;
import cz.muni.fi.pa165.exception.BandManagerNotFoundException;
import cz.muni.fi.pa165.mapper.BandManagerMapper;
import cz.muni.fi.pa165.model.BandManagerDTO;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.when;

class BandManagerServiceTest {
    @Mock
    private BandManagerRepository bandManagerRepository;

    @Mock
    private BandManagerMapper bandManagerMapper;

    private BandManagerServiceImpl bandManagerService;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        bandManagerService = new BandManagerServiceImpl(bandManagerRepository, bandManagerMapper);
    }

    @Test
    void testGetBandManagerById() {
        long id = 1L;
        BandManager bandManager = new BandManager();
        bandManager.setId(id);
        when(bandManagerRepository.findById(id)).thenReturn(Optional.of(bandManager));
        BandManagerDTO expected = new BandManagerDTO();
        expected.setId(id);
        when(bandManagerMapper.BandManagerToBandManagerDTO(bandManager)).thenReturn(expected);

        BandManagerDTO result = bandManagerService.getBandManagerByID(id);
        assertEquals(expected, result);
        Mockito.verify(bandManagerRepository).findById(id);
        Mockito.verify(bandManagerMapper).BandManagerToBandManagerDTO(bandManager);
    }

    @Test
    void testGetNonExistingBandManagerById() {
        long id = 1L;
        when(bandManagerRepository.findById(id)).thenReturn(Optional.empty());

        assertThrows(BandManagerNotFoundException.class, () -> {
            bandManagerService.getBandManagerByID(id);
        });

        Mockito.verify(bandManagerRepository).findById(id);
    }

    @Test
    void updateBandManagerById() {
        long id = 1L;
        BandManagerDTO input = BandManagerDTO.builder()
                .id(id)
                .nickname("Old Nickname")
                .userId(1)
                .build();

        BandManager existing = new BandManager();
        existing.setId(id);
        when(bandManagerRepository.findById(id)).thenReturn(Optional.of(existing));

        BandManagerDTO updatedDTO = BandManagerDTO.builder()
                .id(id)
                .nickname("New Nickname")
                .userId(1)
                .build();
        BandManager updated = bandManagerMapper.BandManagerDTOToBandManager(updatedDTO);
        when(bandManagerMapper.BandManagerDTOToBandManager(input)).thenReturn(updated);
        when(bandManagerMapper.BandManagerToBandManagerDTO(updated)).thenReturn(updatedDTO);

        BandManagerDTO result = bandManagerService.updateBandManagerByID(input);

        assertEquals(updatedDTO, result);
        Mockito.verify(bandManagerRepository).findById(id);
        Mockito.verify(bandManagerMapper).BandManagerDTOToBandManager(input);
        Mockito.verify(bandManagerMapper).BandManagerToBandManagerDTO(updated);
        Mockito.verify(bandManagerRepository).save(updated);


    }

    @Test
    void testRemoveNonExistingBandManager() {
        long id = 1L;
        when(bandManagerRepository.findById(id)).thenReturn(Optional.empty());

        assertThrows(BandManagerNotFoundException.class, () -> {
            bandManagerService.removeBandManagerByID(id);
        });

        Mockito.verify(bandManagerRepository).findById(id);
        Mockito.verify(bandManagerRepository, Mockito.times(0)).delete(Mockito.any(BandManager.class));
    }

    @Test
    void testRemoveBandManagerById() {
        long id = 1L;
        BandManager bandManager = new BandManager();
        bandManager.setId(id);
        when(bandManagerRepository.findById(id)).thenReturn(Optional.of(bandManager));

        bandManagerService.removeBandManagerByID(id);

        Mockito.verify(bandManagerRepository).findById(id);
        Mockito.verify(bandManagerRepository).delete(bandManager);
    }


}