package cz.muni.fi.pa165.service;

import cz.muni.fi.pa165.data.entity.Proposal;
import cz.muni.fi.pa165.data.repository.ProposalRepository;
import cz.muni.fi.pa165.exception.ProposalNotFoundException;
import cz.muni.fi.pa165.mapper.ProposalMapper;
import cz.muni.fi.pa165.model.ProposalDTO;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.when;
import static org.mockito.Mockito.any;


class ProposalServiceImplTest {

    @Mock
    private ProposalRepository proposalRepository;
    @Mock
    private ProposalMapper proposalMapper;

    private ProposalServiceImpl proposalService;
    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        proposalService = new ProposalServiceImpl(proposalRepository, proposalMapper);
    }

    @Test
    void testGetProposalById() {
        long id = 1L;
        Proposal proposal = new Proposal();
        proposal.setId(id);
        when(proposalRepository.findById(id)).thenReturn(Optional.of(proposal));

        ProposalDTO expected = new ProposalDTO();
        expected.setId(id);
        when(proposalMapper.proposalToProposalDTO(proposal)).thenReturn(expected);

        ProposalDTO result = proposalService.getProposalById(id);

        assertEquals(expected, result);
        Mockito.verify(proposalRepository).findById(id);
        Mockito.verify(proposalMapper).proposalToProposalDTO(proposal);
    }
    @Test
    void testGetNonExistingProposalById() {
        long id = 1L;
        when(proposalRepository.findById(id)).thenReturn(Optional.empty());

        assertThrows(ProposalNotFoundException.class, () -> proposalService.getProposalById(id));

        Mockito.verify(proposalRepository).findById(id);
    }


    @Test
    void testCreateProposal() {
        ProposalDTO proposalDTO = ProposalDTO.builder()
                .bandId(1L)
                .userId(2L)
                .resolved(false)
                .build();

        Proposal proposal = new Proposal();
        proposal.setId(1L);
        proposal.setBandId(proposalDTO.getBandId());
        proposal.setUserId(proposalDTO.getUserId());
        proposal.setResolved(proposalDTO.isResolved());
        when(proposalMapper.proposalDTOToProposal(proposalDTO)).thenReturn(proposal);

        Proposal savedProposal = new Proposal();
        savedProposal.setId(1L);
        savedProposal.setBandId(proposalDTO.getBandId());
        savedProposal.setUserId(proposalDTO.getUserId());
        savedProposal.setResolved(proposalDTO.isResolved());
        when(proposalRepository.save(proposal)).thenReturn(savedProposal);

        ProposalDTO expected = ProposalDTO.builder()
                .id(1L)
                .bandId(proposalDTO.getBandId())
                .userId(proposalDTO.getUserId())
                .resolved(proposalDTO.isResolved())
                .build();
        when(proposalMapper.proposalToProposalDTO(savedProposal)).thenReturn(expected);

        ProposalDTO result = proposalService.createProposal(proposalDTO);

        assertEquals(expected, result);
        Mockito.verify(proposalMapper).proposalDTOToProposal(proposalDTO);
        Mockito.verify(proposalRepository).save(proposal);
        Mockito.verify(proposalMapper).proposalToProposalDTO(savedProposal);
    }

    @Test
    void testRemoveProposalById() {
        long id = 1L;
        Proposal proposal = new Proposal();
        proposal.setId(id);
        when(proposalRepository.findById(id)).thenReturn(Optional.of(proposal));

        proposalService.removeProposalById(id);
        Mockito.verify(proposalRepository).delete(proposal);
    }

    @Test
    void removeNonExistingProposalByIdTest() {
        long id = 1L;
        when(proposalRepository.findById(id)).thenReturn(Optional.empty());
        assertThrows(ProposalNotFoundException.class, () -> proposalService.removeProposalById(id));

    }





}